"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TelemetryUsecases = void 0;
const telemetry_interface_1 = require("../../domain/repositories/telemetry.interface");
const telemetry_interface_2 = require("../../domain/telemetry/telemetry.interface");
const context_service_1 = require("../../infrastructure/services/context/context.service");
const common_1 = require("@nestjs/common");
const uuid_1 = require("uuid");
let TelemetryUsecases = class TelemetryUsecases {
    constructor(telemetryRepository, contextService) {
        this.telemetryRepository = telemetryRepository;
        this.contextService = contextService;
        this.crashReports = [];
        this.maxSize = 10;
        this.metricQueue = [];
        this.maxQueueSize = 10;
        this.flushInterval = 1000 * 5;
        this.catchException();
    }
    async createCrashReport(error, source) {
        try {
            if (!this.isCrashReportEnabled())
                return;
            const crashReport = this.buildCrashReport(error);
            if (this.crashReports.includes(JSON.stringify(crashReport)))
                return;
            if (this.crashReports.length >= this.maxSize) {
                this.crashReports.shift();
            }
            this.crashReports.push(JSON.stringify(crashReport));
            await this.telemetryRepository.createCrashReport(crashReport, source);
        }
        catch (e) { }
        return;
    }
    async sendCrashReport() {
        if (!this.isCrashReportEnabled()) {
            return;
        }
        const crashReport = await this.telemetryRepository.getLastCrashReport();
        if (crashReport && !crashReport.metadata.sentAt) {
            const promises = [
                this.telemetryRepository.sendTelemetryToServer(crashReport.event),
            ];
            const collectorEndpoint = process.env.CORTEX_EXPORTER_OLTP_ENDPOINT;
            if (collectorEndpoint) {
                promises.push(this.telemetryRepository.sendTelemetryToOTelCollector(collectorEndpoint, crashReport));
            }
            await Promise.all(promises);
            await this.telemetryRepository.markLastCrashReportAsSent();
        }
        return;
    }
    async readCrashReports(callback) {
        return this.telemetryRepository.readCrashReports(callback);
    }
    buildCrashReport(error) {
        return {
            message: error.message,
            stack: error.stack,
            payload: {
                modelId: this.contextService.get('modelId') || '',
                endpoint: this.contextService.get('endpoint') || '',
                command: this.contextService.get('command') || '',
            },
            sessionId: this.contextService.get('sessionId') || '',
        };
    }
    isCrashReportEnabled() {
        return process.env.CORTEX_CRASH_REPORT !== '0';
    }
    isMetricsEnabled() {
        return process.env.CORTEX_METRICS !== '0';
    }
    isBenchmarkEnabled() {
        return process.env.CORTEX_BENCHMARK !== '0';
    }
    async catchException() {
        process.on('uncaughtException', async (error) => {
            await this.createCrashReport(error, this.contextService.get('source'));
        });
        process.on('unhandledRejection', async (error) => {
            await this.createCrashReport(error, this.contextService.get('source'));
        });
    }
    async initInterval() {
        this.interval = this.flushMetricQueueInterval();
    }
    async sendEvent(events, source) {
        try {
            if (!this.isMetricsEnabled())
                return;
            const sessionId = this.contextService.get('sessionId') || '';
            const sessionEvents = events.map((event) => ({
                ...event,
                sessionId,
            }));
            await this.telemetryRepository.sendEvent(sessionEvents, source);
        }
        catch (e) { }
    }
    async sendActivationEvent(source) {
        try {
            if (!this.isMetricsEnabled())
                return;
            if (!this.lastActiveAt) {
                const currentData = await this.telemetryRepository.getAnonymizedData();
                this.lastActiveAt = currentData === null || currentData === void 0 ? void 0 : currentData.lastActiveAt;
            }
            const isActivatedToday = this.lastActiveAt &&
                new Date(this.lastActiveAt).getDate() === new Date().getDate();
            if (isActivatedToday)
                return;
            const isNewActivation = !this.lastActiveAt;
            await this.sendEvent([
                {
                    name: isNewActivation ? telemetry_interface_2.EventName.NEW_ACTIVATE : telemetry_interface_2.EventName.ACTIVATE,
                },
            ], source);
            this.lastActiveAt = new Date().toISOString();
        }
        catch (e) { }
        await this.updateAnonymousData(this.lastActiveAt);
    }
    async addEventToQueue(event) {
        if (!this.isMetricsEnabled())
            return;
        this.metricQueue.push({
            ...event,
            sessionId: this.contextService.get('sessionId') || '',
        });
        if (this.metricQueue.length >= this.maxQueueSize) {
            await this.flushMetricQueue();
        }
    }
    async flushMetricQueue() {
        if (this.metricQueue.length > 0) {
            clearInterval(this.interval);
            await this.sendEvent(this.metricQueue, telemetry_interface_2.TelemetrySource.CORTEX_SERVER);
            this.interval = this.flushMetricQueueInterval();
            this.metricQueue = [];
        }
    }
    flushMetricQueueInterval() {
        return setInterval(() => {
            this.flushMetricQueue();
        }, this.flushInterval);
    }
    async updateAnonymousData(lastActiveAt) {
        try {
            const currentData = await this.telemetryRepository.getAnonymizedData();
            const updatedData = {
                ...currentData,
                sessionId: (currentData === null || currentData === void 0 ? void 0 : currentData.sessionId) || (0, uuid_1.v4)(),
                ...(lastActiveAt && { lastActiveAt }),
            };
            await this.telemetryRepository.updateAnonymousData(updatedData);
            return updatedData;
        }
        catch (e) {
            return null;
        }
    }
    async sendBenchmarkEvent({ hardware, results, metrics, model, }) {
        try {
            if (!this.isBenchmarkEnabled())
                return;
            const sessionId = this.contextService.get('sessionId') || '';
            await this.telemetryRepository.sendBenchmarkToServer({
                hardware,
                results,
                metrics,
                model,
                sessionId,
            });
        }
        catch (e) { }
    }
};
exports.TelemetryUsecases = TelemetryUsecases;
exports.TelemetryUsecases = TelemetryUsecases = __decorate([
    (0, common_1.Injectable)({ scope: common_1.Scope.TRANSIENT }),
    __param(0, (0, common_1.Inject)('TELEMETRY_REPOSITORY')),
    __metadata("design:paramtypes", [telemetry_interface_1.TelemetryRepository,
        context_service_1.ContextService])
], TelemetryUsecases);
//# sourceMappingURL=telemetry.usecases.js.map