"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getGpuInfo = exports.getCudaVersion = exports.checkNvidiaGPUExist = exports.cudaVersion = void 0;
const child_process_1 = require("child_process");
const fs_1 = require("fs");
const path_1 = require("path");
const app_path_1 = require("./app-path");
const cudaVersion = async () => {
    let filesCuda12;
    let filesCuda11;
    let paths;
    if (process.platform === 'win32') {
        filesCuda12 = ['cublas64_12.dll', 'cudart64_12.dll', 'cublasLt64_12.dll'];
        filesCuda11 = ['cublas64_11.dll', 'cudart64_110.dll', 'cublasLt64_11.dll'];
        paths = process.env.PATH ? process.env.PATH.split(path_1.delimiter) : [];
    }
    else {
        filesCuda12 = ['libcudart.so.12', 'libcublas.so.12', 'libcublasLt.so.12'];
        filesCuda11 = ['libcudart.so.11.0', 'libcublas.so.11', 'libcublasLt.so.11'];
        paths = process.env.LD_LIBRARY_PATH
            ? process.env.LD_LIBRARY_PATH.split(path_1.delimiter)
            : [];
        paths.push('/usr/lib/x86_64-linux-gnu/');
    }
    if (filesCuda12.every((file) => (0, fs_1.existsSync)(file) || (0, app_path_1.checkFileExistenceInPaths)(file, paths)))
        return '12';
    if (filesCuda11.every((file) => (0, fs_1.existsSync)(file) || (0, app_path_1.checkFileExistenceInPaths)(file, paths)))
        return '11';
    return undefined;
};
exports.cudaVersion = cudaVersion;
const checkNvidiaGPUExist = () => {
    return new Promise((resolve) => {
        (0, child_process_1.exec)('nvidia-smi', (error) => {
            if (error) {
                console.log('NVIDIA GPU not detected or nvidia-smi not installed.');
                resolve(false);
            }
            else {
                console.log('NVIDIA GPU detected.');
                resolve(true);
            }
        });
    });
};
exports.checkNvidiaGPUExist = checkNvidiaGPUExist;
const getCudaVersion = () => {
    return new Promise((resolve, reject) => {
        (0, child_process_1.exec)('nvidia-smi', (error, stdout) => {
            if (!error) {
                const cudaVersionLine = stdout.split('\n').find(line => line.includes('CUDA Version'));
                if (cudaVersionLine) {
                    const cudaVersionMatch = cudaVersionLine.match(/CUDA Version:\s+(\d+\.\d+)/);
                    if (cudaVersionMatch) {
                        const cudaVersion = cudaVersionMatch[1];
                        resolve(cudaVersion);
                    }
                    else {
                        reject('CUDA Version not found.');
                    }
                }
                else {
                    reject('CUDA Version not found.');
                }
            }
            else {
                reject(error);
            }
        });
    });
};
exports.getCudaVersion = getCudaVersion;
const getGpuInfo = async () => new Promise((resolve) => {
    (0, child_process_1.exec)('nvidia-smi --query-gpu=index,memory.total,name --format=csv,noheader,nounits', async (error, stdout) => {
        if (!error) {
            let highestVram = 0;
            let highestVramId = '0';
            const gpus = stdout
                .trim()
                .split('\n')
                .map((line) => {
                let [id, vram, name] = line.split(', ');
                const arch = getGpuArch(name);
                vram = vram.replace(/\r/g, '');
                if (parseFloat(vram) > highestVram) {
                    highestVram = parseFloat(vram);
                    highestVramId = id;
                }
                return { id, vram, name, arch };
            });
            resolve(gpus);
        }
        else {
            resolve([]);
        }
    });
});
exports.getGpuInfo = getGpuInfo;
const getGpuArch = (gpuName) => {
    if (!gpuName.toLowerCase().includes('nvidia'))
        return 'unknown';
    if (gpuName.includes('30'))
        return 'ampere';
    else if (gpuName.includes('40'))
        return 'ada';
    else
        return 'unknown';
};
//# sourceMappingURL=cuda.js.map