"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.guessPromptTemplateFromHuggingFace = guessPromptTemplateFromHuggingFace;
exports.fetchHuggingFaceRepoData = fetchHuggingFaceRepoData;
exports.fetchJanRepoData = fetchJanRepoData;
exports.getRepoModelsUrl = getRepoModelsUrl;
exports.getHFModelMetadata = getHFModelMetadata;
const huggingface_1 = require("../infrastructure/constants/huggingface");
const prompt_constants_1 = require("../infrastructure/constants/prompt-constants");
const gguf_1 = require("@huggingface/gguf");
const axios_1 = __importDefault(require("axios"));
const normalize_model_id_1 = require("./normalize-model-id");
function guessPromptTemplateFromHuggingFace(jinjaCode) {
    if (!jinjaCode) {
        console.log('No jinja code provided. Returning default LLAMA_2');
        return prompt_constants_1.LLAMA_2;
    }
    if (typeof jinjaCode !== 'string') {
        console.log(`Invalid jinja code provided (type is ${typeof jinjaCode}). Returning default LLAMA_2`);
        return prompt_constants_1.LLAMA_2;
    }
    switch (jinjaCode) {
        case prompt_constants_1.ZEPHYR_JINJA:
            return prompt_constants_1.ZEPHYR;
        case prompt_constants_1.OPEN_CHAT_3_5_JINJA:
            return prompt_constants_1.OPEN_CHAT_3_5;
        case prompt_constants_1.LLAMA_3_JINJA:
            return prompt_constants_1.LLAMA_3;
        default:
            return prompt_constants_1.LLAMA_2;
    }
}
async function fetchHuggingFaceRepoData(repoId) {
    const sanitizedUrl = getRepoModelsUrl(repoId);
    const { data: response } = await axios_1.default.get(sanitizedUrl);
    if (response['error'] != null) {
        throw new Error(response['error']);
    }
    const data = response;
    if (data.tags.indexOf('gguf') === -1) {
        throw `${repoId} is not supported. Only GGUF models are supported.`;
    }
    const url = new URL(sanitizedUrl);
    const paths = url.pathname.split('/').filter((e) => e.trim().length > 0);
    for (let i = 0; i < data.siblings.length; i++) {
        const downloadUrl = (0, huggingface_1.HUGGING_FACE_DOWNLOAD_FILE_MAIN_URL)([paths[2], paths[3]].join('/'), data.siblings[i].rfilename);
        data.siblings[i].downloadUrl = downloadUrl;
    }
    huggingface_1.AllQuantizations.forEach((quantization) => {
        data.siblings.forEach((sibling) => {
            if (!sibling.quantization && sibling.rfilename.includes(quantization)) {
                sibling.quantization = quantization;
            }
        });
    });
    data.modelUrl = (0, huggingface_1.HUGGING_FACE_REPO_URL)(paths[2], paths[3]);
    return data;
}
async function fetchJanRepoData(modelId) {
    var _a;
    const repo = modelId.split(':')[0];
    const tree = await (0, normalize_model_id_1.parseModelHubEngineBranch)((_a = modelId.split(':')[1]) !== null && _a !== void 0 ? _a : (!modelId.includes('/') ? 'default' : ''));
    const url = getRepoModelsUrl(`${!modelId.includes('/') ? 'cortexso/' : ''}${repo}`, tree);
    const res = await fetch(url);
    const jsonData = await res.json();
    if ('siblings' in jsonData) {
        huggingface_1.AllQuantizations.forEach((quantization) => {
            jsonData.siblings.forEach((sibling) => {
                if (!sibling.quantization && sibling.rfilename.includes(quantization)) {
                    sibling.quantization = quantization;
                    sibling.downloadUrl = (0, huggingface_1.HUGGING_FACE_DOWNLOAD_FILE_MAIN_URL)(repo, sibling.rfilename);
                }
            });
        });
        return jsonData;
    }
    const response = jsonData;
    if ('error' in response && response.error != null) {
        throw new Error(response.error);
    }
    const data = {
        siblings: Array.isArray(response)
            ? response.map((e) => {
                var _a;
                return {
                    rfilename: e.path,
                    downloadUrl: (0, huggingface_1.HUGGING_FACE_TREE_REF_URL)(repo, tree, e.path),
                    fileSize: (_a = e.size) !== null && _a !== void 0 ? _a : 0,
                };
            })
            : [],
        tags: ['gguf'],
        id: modelId,
        modelId: modelId,
        author: 'cortexso',
        sha: '',
        downloads: 0,
        lastModified: '',
        private: false,
        disabled: false,
        gated: false,
        pipeline_tag: 'text-generation',
        cardData: {},
        createdAt: '',
    };
    huggingface_1.AllQuantizations.forEach((quantization) => {
        data.siblings.forEach((sibling) => {
            if (!sibling.quantization && sibling.rfilename.includes(quantization)) {
                sibling.quantization = quantization;
            }
        });
    });
    data.modelUrl = url;
    return data;
}
function getRepoModelsUrl(repoId, tree) {
    return `${(0, huggingface_1.HUGGING_FACE_REPO_MODEL_API_URL)(repoId)}${tree ? `/tree/${tree}` : ''}`;
}
async function getHFModelMetadata(ggufUrl) {
    var _a;
    try {
        const { metadata } = await (0, gguf_1.gguf)(ggufUrl);
        const index = metadata['tokenizer.ggml.eos_token_id'];
        const hfChatTemplate = metadata['tokenizer.chat_template'];
        const promptTemplate = guessPromptTemplateFromHuggingFace(hfChatTemplate);
        const stopWord = (_a = metadata['tokenizer.ggml.tokens'][index]) !== null && _a !== void 0 ? _a : '';
        const version = metadata['version'];
        return {
            stopWord,
            promptTemplate,
            version,
        };
    }
    catch (err) {
        console.log('Failed to get model metadata:', err.message);
        return undefined;
    }
}
//# sourceMappingURL=huggingface.js.map